@@ -6,7 +6,6 @@ from multiprocessing import Pool
 
 import comet_maths as cm
 import numpy as np
-
 import punpy.utilities.utilities as util
 
 """___Authorship___"""
@@ -18,7 +17,9 @@ __status__ = "Development"
 
 
 class MCPropagation:
-    def __init__(self, steps, parallel_cores=0, dtype=None, verbose=False):
+    def __init__(
+        self, steps, parallel_cores=0, dtype=None, verbose=False, MCdimlast=True
+    ):
         """
         Initialise MC Propagator
 
@@ -30,6 +31,8 @@ class MCPropagation:
         :type dtype: numpy dtype
         :param verbose: bool to set if logging info should be printed
         :type verbose: bool
+        :param MCdimlast: bool to set whether the MC dimension should be moved to the last dimension when running through the measurment function (when parallel_cores==0). This can be useful for broadcasting within the measurement function. defaults to False
+        :type MCdimlast: bool
         """
 
         self.MCsteps = steps
@@ -39,6 +42,7 @@ class MCPropagation:
             self.pool = Pool(self.parallel_cores)
         self.verbose = verbose
         self.starttime = time.time()
+        self.MCdimlast = MCdimlast
 
     def propagate_random(
         self,
@@ -59,6 +63,7 @@ class MCPropagation:
         refyvar=0,
         pdf_shape="gaussian",
         pdf_params=None,
+        allow_some_nans=True,
     ):
         """
         Propagate random uncertainties through measurement function with n input quantities.
@@ -92,14 +97,16 @@ class MCPropagation:
         :type fixed_corr_var: bool or integer, optional
         :param output_vars: number of output parameters in the measurement function. Defaults to 1.
         :type output_vars: integer, optional
-        :param PD_corr: set to True to make sure returned correlation matrices are positive semi-definite, default to True
+        :param PD_corr: set to True to make sure returned correlation matrices are positive semi-definite, default to True.
         :type PD_corr: bool, optional
-        :param refyvar: Index of output variable with reference shape (only relevant when output_vars>1; should be output variable with most dimensions; affects things like repeat_dims)
+        :param refyvar: Index of output variable with reference shape (only relevant when output_vars>1; should be output variable with most dimensions; affects things like repeat_dims).
         :type refyvar: int, optional
-        :param pdf_shape: string identifier of the probability density function shape, defaults to gaussian
+        :param pdf_shape: string identifier of the probability density function shape, defaults to gaussian.
         :type pdf_shape: str, optional
-        :param pdf_params: dictionaries defining optional additional parameters that define the probability density function, Defaults to None (gaussian does not require additional parameters)
+        :param pdf_params: dictionaries defining optional additional parameters that define the probability density function, Defaults to None (gaussian does not require additional parameters).
         :type pdf_params: dict, optional
+        :param allow_some_nans: set to False to ignore any MC sample which has any nan's in the measurand. Defaults to True, in which case only MC samples with only nan's are ignored.
+        :type allow_some_nans: bool, optional
         :return: uncertainties on measurand
         :rtype: array
         """
@@ -127,6 +134,7 @@ class MCPropagation:
             refyvar=refyvar,
             pdf_shape=pdf_shape,
             pdf_params=pdf_params,
+            allow_some_nans=allow_some_nans,
         )
 
     def propagate_systematic(
@@ -148,6 +156,7 @@ class MCPropagation:
         refyvar=0,
         pdf_shape="gaussian",
         pdf_params=None,
+        allow_some_nans=True,
     ):
         """
         Propagate systematic uncertainties through measurement function with n input quantities.
@@ -189,6 +198,8 @@ class MCPropagation:
         :type pdf_shape: str, optional
         :param pdf_params: dictionaries defining optional additional parameters that define the probability density function, Defaults to None (gaussian does not require additional parameters)
         :type pdf_params: dict, optional
+        :param allow_some_nans: set to False to ignore any MC sample which has any nan's in the measurand. Defaults to True, in which case only MC samples with only nan's are ignored.
+        :type allow_some_nans: bool, optional
         :return: uncertainties on measurand
         :rtype: array
         """
@@ -216,6 +227,7 @@ class MCPropagation:
             refyvar=refyvar,
             pdf_shape=pdf_shape,
             pdf_params=pdf_params,
+            allow_some_nans=allow_some_nans,
         )
 
     def propagate_cov(
@@ -236,6 +248,7 @@ class MCPropagation:
         refyvar=0,
         pdf_shape="gaussian",
         pdf_params=None,
+        allow_some_nans=True,
     ):
         """
         Propagate uncertainties with given covariance matrix through measurement function with n input quantities.
@@ -276,6 +289,8 @@ class MCPropagation:
         :type pdf_shape: str, optional
         :param pdf_params: dictionaries defining optional additional parameters that define the probability density function, Defaults to None (gaussian does not require additional parameters)
         :type pdf_params: dict, optional
+        :param allow_some_nans: set to False to ignore any MC sample which has any nan's in the measurand. Defaults to True, in which case only MC samples with only nan's are ignored.
+        :type allow_some_nans: bool, optional
         :return: uncertainties on measurand
         :rtype: array
         """
@@ -305,6 +320,7 @@ class MCPropagation:
             refyvar=refyvar,
             pdf_shape=pdf_shape,
             pdf_params=pdf_params,
+            allow_some_nans=allow_some_nans,
         )
 
     def propagate_standard(
@@ -326,6 +342,7 @@ class MCPropagation:
         refyvar=0,
         pdf_shape="gaussian",
         pdf_params=None,
+        allow_some_nans=True,
     ):
         """
         Propagate uncertainties through measurement function with n input quantities. Correlations must be specified in corr_x.
@@ -367,6 +384,8 @@ class MCPropagation:
         :type pdf_shape: str, optional
         :param pdf_params: dictionaries defining optional additional parameters that define the probability density function, Defaults to None (gaussian does not require additional parameters)
         :type pdf_params: dict, optional
+        :param allow_some_nans: set to False to ignore any MC sample which has any nan's in the measurand. Defaults to True, in which case only MC samples with only nan's are ignored.
+        :type allow_some_nans: bool, optional
         :return: uncertainties on measurand
         :rtype: array
         """
@@ -453,7 +472,6 @@ class MCPropagation:
                 xb, u_xb = util.select_repeated_x(
                     x, u_x, param_fixed, i, repeat_dims, repeat_shape
                 )
-
                 out_i = self.propagate_standard(
                     func,
                     xb,
@@ -521,7 +539,9 @@ class MCPropagation:
                     pdf_params=pdf_params,
                 )
 
-            MC_y = self.run_samples(func, MC_x, output_vars=output_vars)
+            MC_y = self.run_samples(
+                func, MC_x, output_vars=output_vars, allow_some_nans=allow_some_nans
+            )
 
             return self.process_samples(
                 MC_x,
@@ -578,6 +598,7 @@ class MCPropagation:
                 pdf_params=pdf_params,
                 comp_list=comp_list,
             )
+
         if corr_between is not None:
             MC_data = cm.correlate_sample_corr(MC_data, corr_between)
         if self.verbose:
@@ -585,7 +606,6 @@ class MCPropagation:
                 "samples generated (%s s since creation of prop object)"
                 % (time.time() - self.starttime)
             )
-
         return MC_data
 
     def generate_MC_sample_cov(
@@ -896,6 +916,19 @@ class MCPropagation:
 
         shapewarning = False
         for i in range(len(x)):
+            try:
+                if not (np.all(np.isfinite(u_x[i]))):
+                    warnings.warn(
+                        "One of your uncertainties has nans (of inf). This can cause issues in the uncertainty propagation. Please remove or replace your nans."
+                    )
+
+                if not (np.all(np.isfinite(corr_x[i]))):
+                    warnings.warn(
+                        "One of your error correlation matrices has nans (of inf). This can cause issues in the uncertainty propagation. Please remove or replace your nans."
+                    )
+            except:
+                pass
+
             if hasattr(x[i], "__shape__"):
                 if param_fixed is not None:
                     if (
@@ -1058,11 +1091,13 @@ class MCPropagation:
                 outs[0] = np.empty(
                     (
                         output_vars,
-                        n_repeats,
-                    )
-                    + out_0[0].shape,
-                    dtype=self.dtype,
+                    ),
+                    dtype=object,
                 )
+                for i in range(output_vars):
+                    outs[0][i]=np.empty((n_repeats,
+                                         )
+                                        + out_0[i].shape,dtype=self.dtype)
 
             # elif (out_0[0][0].ndim) > 1:
             #     outs[0] = np.empty((output_vars,len(out_0[0][0]),n_repeats,)+out_0[0][0].shape)
@@ -1071,11 +1106,14 @@ class MCPropagation:
                 outs[0] = np.empty(
                     (
                         output_vars,
-                        n_repeats,
-                    )
-                    + out_0[0][0].shape,
-                    dtype=self.dtype,
+                    ),
+                    dtype=object,
                 )
+                for i in range(output_vars):
+                    outs[0][i]=np.empty((n_repeats,
+                                         )
+                                        + out_0[0][i].shape,dtype=self.dtype)
+
 
             extra_index = 0
             if return_corr:
@@ -1264,8 +1302,7 @@ class MCPropagation:
         if not return_corr and output_vars == 1 and not return_samples:
             u_func = np.array(outs, dtype=self.dtype)
         else:
-            u_func = np.array(outs[0], dtype=self.dtype)
-
+            u_func = np.array(outs[0])
         if len(repeat_dims) == 1:
             if output_vars == 1:
                 u_func = np.squeeze(np.moveaxis(u_func, 0, repeat_dims[0]))
@@ -1363,7 +1400,7 @@ class MCPropagation:
         start=None,
         end=None,
         sli=None,
-        broadcasting=True,
+        allow_some_nans=True,
     ):
         """
         process all the MC samples of input quantities through the measurand function
@@ -1372,8 +1409,16 @@ class MCPropagation:
         :type func: function
         :param MC_x: MC-generated samples of input quantities
         :type MC_x: array[array]
-        :param yshapes: shape of the measurand(s)
-        :type yshapes: tuple or list of tuples
+        :param output_vars: number of output parameters in the measurement function. Defaults to 1.
+        :type output_vars: integer, optional
+        :param start: set this parameter to propagate the input quantities through the measurement function starting from a specfic index. All input quantities before this index are ignored. Defaults to None, in which case no input quantities are ignored.
+        :type start: integer, optional
+        :param end: set this parameter to propagate the input quantities through the measurement function up until a specfic index. All input quantities after this index are ignored. Defaults to None, in which case no input quantities are ignored.
+        :type end: integer, optional
+        :param sli: set this parameter to a slice to set which input quantities will be processed through the measurment function. All other input quantities are ignored. Can only be used if start and end are not set. Defaults to None, in which case no input quantities are ignored.
+        :type sli: slice, optional
+        :param allow_some_nans: set to False to ignore any MC sample which has any nan's in the measurand. Defaults to True, in which case only MC samples with only nan's are ignored.
+        :type allow_some_nans: bool, optional
         :return: MC sample of measurand
         :rtype: array[array]
         """
@@ -1385,12 +1430,12 @@ class MCPropagation:
             indices = range(*sli.indices(self.MCsteps))
 
         if self.parallel_cores == 0:
-            if broadcasting:
-                return np.moveaxis(
+            if self.MCdimlast:
+                MC_y = np.moveaxis(
                     func(*[np.moveaxis(dat[sli], 0, -1) for dat in MC_x]), -1, 0
                 )
             else:
-                return func(*[x[sli] for x in MC_x])
+                MC_y = func(*[x[sli] for x in MC_x])
 
         elif self.parallel_cores == 1:
             MC_y = list(map(func, *[x[sli] for x in MC_x]))
@@ -1401,13 +1446,106 @@ class MCPropagation:
                 MC_x2[i] = [MC_x[j][index, ...] for j in range(len(MC_x))]
             MC_y = self.pool.starmap(func, MC_x2)
 
-        MC_y=np.array([MC_y[i] for i in range(len(indices)) if MC_y[i] is not None],dtype=self.dtype)
+        # for i in range(len(indices)):
+        #     print(MC_y[i])
+        #     print([MC_x[j][i] for j in range(len(MC_x))])
+        #     print(output_vars,MC_y[i].shape)
+        #     print([np.any(np.isfinite(MC_y[i][ivar])) for ivar in range(output_vars)])
+        #     print(np.all([np.any(np.isfinite(MC_y[i][ivar])) for ivar in range(output_vars)]),)
+        #     print(np.isfinite(MC_y[i]))
 
-        if len(MC_y)<len(indices):
-            print(
-                "%s of the %s MC samples were not processed correctly (returned None) and will be ignored in the punpy output"
-                % (len(indices)-len(MC_y),len(indices))
-            )
+        if output_vars == 1:
+            if allow_some_nans:
+                MC_y = np.array(
+                    [
+                        MC_y[i]
+                        for i in range(len(indices))
+                        if np.any(np.isfinite(MC_y[i]))
+                    ],
+                    dtype=self.dtype,
+                )
+            else:
+                MC_y = np.array(
+                    [
+                        MC_y[i]
+                        for i in range(len(indices))
+                        if np.all(np.isfinite(MC_y[i]))
+                    ],
+                    dtype=self.dtype,
+                )
+        else:
+            if allow_some_nans:
+                try:
+                    MC_y = np.array(
+                        [
+                            MC_y[i]
+                            for i in range(len(indices))
+                            if np.all(
+                                [
+                                    np.any(np.isfinite(MC_y[i][ivar]))
+                                    for ivar in range(output_vars)
+                                ]
+                            )
+                        ],
+                        dtype=self.dtype,
+                    )
+                except:
+                    MC_y = np.array(
+                        [
+                            MC_y[i]
+                            for i in range(len(indices))
+                            if np.all(
+                                [
+                                    np.any(np.isfinite(MC_y[i][ivar]))
+                                    for ivar in range(output_vars)
+                                ]
+                            )
+                        ],
+                        dtype=object,
+                    )
+
+            else:
+                try:
+                    MC_y = np.array(
+                        [
+                            MC_y[i]
+                            for i in range(len(indices))
+                            if np.all(
+                                [
+                                    np.all(np.isfinite(MC_y[i][ivar]))
+                                    for ivar in range(output_vars)
+                                ]
+                            )
+                        ],
+                        dtype=self.dtype,
+                    )
+                except:
+                    MC_y = np.array(
+                        [
+                            MC_y[i]
+                            for i in range(len(indices))
+                            if np.all(
+                                [
+                                    np.all(np.isfinite(MC_y[i][ivar]))
+                                    for ivar in range(output_vars)
+                                ]
+                            )
+                        ],
+                        dtype=object,
+                    )
+
+        if len(MC_y) < len(indices):
+            if allow_some_nans:
+                print(
+                    "%s of the %s MC samples were not processed correctly (contained only nans) and will be ignored in the punpy output"
+                    % (len(indices) - len(MC_y), len(indices))
+                )
+
+            else:
+                print(
+                    "%s of the %s MC samples were not processed correctly (contained some nans) and will be ignored in the punpy output"
+                    % (len(indices) - len(MC_y), len(indices))
+                )
 
         if self.verbose:
             print(
@@ -1459,6 +1597,19 @@ class MCPropagation:
 
         # if hasattr(MC_y[0,0], '__len__'):
         #     print(yshape,np.array(MC_y[0,0]).shape,np.array(MC_y[1,0]).shape,np.array(MC_y[2,0]).shape,np.array(MC_y[3,0]).shape)
+        if yshapes is None:
+            if output_vars>1:
+                yshapes=[MC_y[0][i].shape for i in range(output_vars)]
+            else:
+                yshapes=MC_y[0].shape
+
+        if len(MC_y) == 0:
+            if output_vars == 1:
+                u_func = np.nan * np.zeros(yshapes[0])
+            else:
+                u_func = np.empty(output_vars, dtype=object)
+                for i in range(output_vars):
+                    u_func[i] = np.nan * np.zeros(yshapes[i])
         if output_vars == 1:
             u_func = np.std(MC_y, axis=0, dtype=self.dtype)
 
@@ -1496,12 +1647,9 @@ class MCPropagation:
         else:
             if output_vars == 1:
                 if fixed_corr is None:
-                    corr_y = cm.calculate_corr(MC_y, corr_dims).astype(self.dtype)
-                    if PD_corr:
-                        if not cm.isPD(corr_y):
-                            corr_y = cm.nearestPD_cholesky(
-                                corr_y, corr=True, return_cholesky=False
-                            )
+                    corr_y = cm.calculate_corr(
+                        MC_y, corr_dims, PD_corr, dtype=self.dtype
+                    )
                 else:
                     corr_y = fixed_corr
 
@@ -1522,18 +1670,14 @@ class MCPropagation:
                 for i in range(output_vars):
                     if fixed_corr is None:
                         if complex_shapes:
-                            corr_ys[i] = cm.calculate_corr(MC_y2[i], corr_dims).astype(
-                                self.dtype
+                            corr_ys[i] = cm.calculate_corr(
+                                MC_y2[i], corr_dims, PD_corr, self.dtype
                             )
                         else:
                             corr_ys[i] = cm.calculate_corr(
-                                MC_y[:, i], corr_dims
-                            ).astype(self.dtype)
-                        if PD_corr:
-                            if not cm.isPD(corr_ys[i]):
-                                corr_ys[i] = cm.nearestPD_cholesky(
-                                    corr_ys[i], corr=True, return_cholesky=False
-                                )
+                                MC_y[:, i], corr_dims, PD_corr, self.dtype
+                            )
+
                     else:
                         corr_ys[i] = fixed_corr
 
@@ -1545,18 +1689,14 @@ class MCPropagation:
 
                     corr_out = np.mean(
                         [
-                            cm.calculate_corr(MC_y2[:, :, i]).astype(self.dtype)
+                            cm.calculate_corr(
+                                MC_y2[:, :, i], PD_corr=PD_corr, dtype=self.dtype
+                            )
                             for i in range(len(MC_y2[0, 0]))
                         ],
                         axis=0,
                     )
 
-                if PD_corr and corr_out is not None:
-                    if not cm.isPD(corr_out):
-                        corr_out = cm.nearestPD_cholesky(
-                            corr_out, corr=True, return_cholesky=False
-                        )
-
                 if self.verbose:
                     print(
                         "corr calculated for output_var>1 (%s s since creation of prop object)"
